Package: DeMAND
Title: DeMAND
Version: 1.42.0
Date: 2015-07-13
Author: Jung Hoon Woo <jw2853@columbia.edu>, Yishai Shimoni <ys2559@columbia.edu>
Maintainer: Jung Hoon Woo <jw2853@columbia.edu>, Mariano Alvarez <reef103@gmail.com>
Description: DEMAND predicts Drug MoA by interrogating a cell context specific regulatory network with a small number (N >= 6) of compound-induced gene expression signatures, to elucidate specific proteins whose interactions in the network is dysregulated by the compound.
Depends: R (>= 2.14.0), KernSmooth, methods
License: file LICENSE
biocViews: SystemsBiology, NetworkEnrichment, GeneExpression,
        StatisticalMethod, Network
git_url: https://git.bioconductor.org/packages/DeMAND
git_branch: RELEASE_3_23
git_last_commit: 3b27aac
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:40:23 UTC; biocbuild
