\name{bcellExp}
\alias{bcellExp}
\docType{data}
\title{
B cell expression data
}
\description{
 subest of a gene expression profiles from DLBCL cells treated by Geldanamycin and by DMSO as control.
}
\keyword{datasets}
